package com.so.net.example;

import com.so.net.DeviceApi;
import com.so.net.DeviceApi.*;
import com.so.net.SignApi;
import static com.so.net.SignApi.Pdf.*;
import java.awt.Color;
import java.util.Optional;
import static com.so.net.SignApi.*;
import com.so.net.SignApi.Behaviour;
import static java.lang.Math.*;
import java.nio.file.Path;

public class Common {

	static class ClickedButton {

		Optional<Button.Kind> kind;
	}

	static Optional<Button.Kind> waitForButton(int timeoutSeconds) throws InterruptedException {
		ClickedButton obj = new ClickedButton();
		DeviceApi.Button.setCallback(e -> {
			obj.kind = Optional.of(e.kind);
			notify(obj);
		});
		wait(obj, timeoutSeconds);
		return obj.kind;
	}

	static void waitForStartSignButton(int timeoutSeconds) throws InterruptedException {
		Object obj = new Object();
		DeviceApi.Button.setCallback(e -> {
			if (DeviceApi.Button.Kind.StartSignature == e.kind) {
				notify(obj);
			}
		});
		wait(obj, timeoutSeconds);
	}

	static void wait(Object obj, int timeoutSeconds) throws InterruptedException {
		synchronized (obj) {
			obj.wait(timeoutSeconds * 1000);
		}
	}

	static void notify(Object obj) {
		synchronized (obj) {
			obj.notifyAll();
		}
	}

	static boolean exampleSign() {
		Behaviour behaviour = Behaviour.Builder.getDefault()
			.signInDocument(Behaviour.DocSign.Rectangle.Builder
				// The rectangle should be at least 40mm wide and 30mm high.
				// and place the signature rectangle on the device display
				// with a distance of 10mm to the lower and the left display edge.
				.fromBottomLeft(10, 10, 40, 30)
				// The document shall be shown on the pad while signing
				.withColor(Color.LIGHT_GRAY)
				// The rectangle shall be highlighted by a red border
				.withBorderColor(Color.RED)
				// which shall be 5 dots wide.
				.withBorderWidth(5))
			.build().get();

		// Choose a field name, field position and signature info.
		FieldName fieldName = FieldName.of(null);

		// Choose the position of the new signature field.
		int page = 0;
		FieldPosition pageDim = getPageDimensions().get(page);
		// The default length unit is inch/72.
		double width = 4 * 72; //4 inches wide
		double height = 2 * 72; //2 inches high
		double row0 = max(pageDim.getHeight() / 2 - height / 2, 0);
		double column0 = max(pageDim.getWidth() / 2 - width / 2, 0);
		double row1 = min(row0 + height / 2, pageDim.getHeight());
		double column1 = min(column0 + width / 2, pageDim.getWidth());
		FieldPosition fieldPosition = FieldPosition.of(row0, column0, row1, column1, page);

		// Adds a signature field with the given field name and position.
		addSignatureField(fieldName, fieldPosition);

		// This queries for the signature field just added.
		SignatureField signatureField = getSignatureField(fieldName).get();

		// A signature field can have these entries.
		// They are not mandatory. Pass an empty string or null to skip a field.
		final String name = "Signer's name.";
		final String reason = "Reason for signing.";
		final String location = "The location.";
		final String contactInfo = "The contact info.";
		SignatureInfo signatureInfo = SignatureInfo.of(name, reason, location, contactInfo);

		// Signs this signature field with the selected pad.
		return sign(signatureField, signatureInfo, behaviour);
	}

	static Optional<SignApi.Pdf.SignatureField> getSignatureField(SignApi.Pdf.FieldName fieldName) {
		return getAcroFormFields().stream()
			.filter(field -> fieldName.equals(field.getFieldName()))
			.filter(field -> field instanceof SignApi.Pdf.SignatureField)
			.map(field -> (SignApi.Pdf.SignatureField) field)
			.findFirst();
	}

	static void setTheDriverXmlCertificate(Path certPath) {
		boolean fileExists = certPath.toFile().exists();
		if (!fileExists) {
			System.out.println(certPath.toAbsolutePath().toString() + " does not exist.");
		}
		boolean success = DeviceApi.DriverXmlCertificate.set(certPath);
		if (!success) {
			System.out.println("Failed to set the driver XML certificate.");
		}
	}

	static void showAndEnableAllButtonsInTheDocViewMode() {
		// The device can show document pages in the DocView mode.
		DeviceApi.Button.Bar.get(DeviceApi.Button.Bar.Mode.DocView)
			.ifPresent(bar -> {
				bar.configs.forEach(Common::showAndEnable);
				bar.set();
			});
	}

	static void showAndEnable(Button.Config buttonConfig) {
		buttonConfig.setEnabled(true);
		buttonConfig.setVisible(true);
	}
}
