package com.so.net.example;

import static com.so.net.DeviceApi.*;
import com.so.net.SignApi;
import static com.so.net.SignApi.Pdf.*;
import static com.so.net.example.Common.*;
import static java.lang.Math.*;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.List;
import java.util.Optional;

public class ViewAPdfWithPad {

	public static void main(String[] args) throws Exception {
		// This requires
		// .NET5 with hostfxr,
		// libgdiplus on Linux and MacOS,
		// the PDF service,
		// and the native SignAPI.
		SignApi.load();

		final List<Device.Name> devices = enumerateDevices(Filter.StepOver);
		if (devices.isEmpty()) {
			System.out.println("Exit because no device was found.");
			return;
		}

		// Select the first device.
		Device.Name firstDevice = devices.get(0);
		setDevice(firstDevice);

		// Loads a PDF document from the file system.
		byte[] theDocument = Files.readAllBytes(Paths.get("TheDocument.pdf"));
		loadPdf(theDocument);

		// View the document on the pad
		// wait and print button clicks
		// and stop if the start signature button was clicked
		// or stop if no button is clicked for more than 10 minutes.
		showAndEnableAllButtonsInTheDocViewMode();
		printButtonClicksInDocViewMode(600, 0);

		// Frees unmanaged resources.
		close();
	}

	static void printButtonClicksInDocViewMode(int timeout, int page) {
		int numberOfPages = getNumberOfPages().orElse(0);

		final int pageInRange = max(0, min(numberOfPages - 1, page));
		renderPage(pageInRange, 96)
			.ifPresent(pageImage -> DocumentViewing.start(pageImage, 1 + pageInRange, numberOfPages));

		Optional<Button.Kind> buttonKind;
		try {
			buttonKind = waitForButton(timeout);
		} catch (InterruptedException ex) {
			return;
		}

		if (buttonKind.isPresent()) {
			Button.Kind kind = buttonKind.get();

			printButton(kind);
			int nextPage = pageInRange;
			if (Button.Kind.Next == kind) {
				nextPage++;
			} else if (Button.Kind.Prev == kind) {
				nextPage--;
			} else if (Button.Kind.StartSignature == kind) {
				System.out.println("Good bye.");
				return;
			}

			printButtonClicksInDocViewMode(timeout, nextPage);
		}
	}

	static void printButton(Button.Kind buttonKind) {
		System.out.println("Button " + buttonKind.toString() + " was clicked.");
	}
}
